/*
 * Copyright (C) 2013 CERN (www.cern.ch)
 * Author: Aurelio Colosimo
 *
 * Released to the public domain
 */

#include <ppsi/ppsi.h>
#include <ppsi-wrs.h>

#define PPSI_EXPORT_STRUCTURES
#include <ppsi_exports.h>


static struct pp_globals *ppg_local;


/* Execute command coming ipc */
static int wrsipc_cmd(int cmd, int value)
{
	if(cmd == PPSIEXP_COMMAND_WR_TRACKING) {
		if ( CONFIG_HAS_EXT_WR ) {
			wrh_servo_enable_tracking(value);
			return 0;
		}
	}
	if(cmd == PPSIEXP_COMMAND_L1SYNC_TRACKING) {
		if ( CONFIG_HAS_EXT_L1SYNC ) {
			wrh_servo_enable_tracking(value);
			return 0;
		}
	}
	return -1;

}

static int export_cmd(const struct minipc_pd *pd,
				 uint32_t *args, void *ret)
{
	int i;
	i = wrsipc_cmd(args[0], args[1]);
	*(int *)ret = i;
	return 0;
}

static int in_range(int val, int min, int max)
{
    if (val < min || val > max)
	return 0;
    return 1;
}

static int update_param_cmd(const struct minipc_pd *pd, uint32_t *args,
			    void *ret)
{
	int rval = PPSIEXP_RET_OK;
	int param_type = args[0];
	int param_val = args[1];

	switch (param_type) {
	case PPSIEXP_PARAM_PRIORITY1_CMD:
		printf("%s: cmd %d (PPSIEXP_PARAM_PRIORITY1_CMD) value %d\n",
		       __func__, param_type, param_val);
		if (!in_range(param_val, PP_MIN_PRIORITY1, PP_MAX_PRIORITY1)) {
		    rval = PPSIEXP_RET_ERROR_VAL;
		    break;
		}
		ppg_local->rt_opts->priority1 = param_val;
		bmc_apply_configured_device_attributes(ppg_local);
		break;
	case PPSIEXP_PARAM_PRIORITY2_CMD:
		printf("%s: cmd %d (PPSIEXP_PARAM_PRIORITY2_CMD) value %d\n",
		       __func__, param_type, param_val);
		if (!in_range(param_val, PP_MIN_PRIORITY2, PP_MAX_PRIORITY2)) {
		    rval = PPSIEXP_RET_ERROR_VAL;
		    break;
		}
		ppg_local->rt_opts->priority2 = param_val;
		bmc_apply_configured_device_attributes(ppg_local);
		break;
	default:
		printf("%s: not found cmd %d value %d\n", __func__, param_type,
		       param_val);
		rval = PPSIEXP_RET_ERROR_CMD;
		break;
	}

	*(int *)ret = rval;
	return 0;
}

static int update_param64_cmd(const struct minipc_pd *pd, uint32_t *args,
			      void *ret)
{
	int rval = PPSIEXP_RET_OK;
	int param_type = args[0];
	int64_t param_val = *((int64_t *)&args[1]);

	switch (param_type) {
	case PPSIEXP_PARAM64_ALPHA_CMD:
		printf("%s: cmd %d (PPSIEXP_PARAM64_ALPHA_CMD) value %lld\n",
		       __func__, param_type, param_val);

		{
			struct pp_instance *ppi;
			int i;

			for (i = 0; i < get_numberPorts(GDSDEF(ppg_local)); i++) {
				ppi = INST(ppg_local, i);
				ppi->asymmetryCorrectionPortDS.scaledDelayCoefficient =
					param_val;
				ppi->portDS->delayAsymCoeff =
					pp_servo_calculateDelayAsymCoefficient(
						ppi->asymmetryCorrectionPortDS.scaledDelayCoefficient);
			}
		}
		break;

	default:
		printf("%s: not found cmd %d value %lld\n",
		       __func__, param_type, param_val);
		rval = PPSIEXP_RET_ERROR_CMD;
		break;
	}

	*(int *)ret = rval;
	return 0;
}


/* To be called at startup, right after the creation of server channel */
void wrs_init_ipcserver(struct pp_globals *ppg, struct minipc_ch *ppsi_ch)
{
	ppg_local = ppg;
	__rpcdef_cmd.f = export_cmd;
	ppsiexp_update_param_cmd.f = update_param_cmd;
	ppsiexp_update_param64_cmd.f = update_param64_cmd;

	minipc_export(ppsi_ch, &__rpcdef_cmd);
	minipc_export(ppsi_ch, &ppsiexp_update_param_cmd);
	minipc_export(ppsi_ch, &ppsiexp_update_param64_cmd);
}
